
def apply_catalog_corrections(root, total_flux='flux_auto', auto_corr=True, get_external_photometry=False, aperture_indices='all', suffix='_apcorr', verbose=True, apply_background=True):
    """
    Aperture and background corrections to photometric catalog
    """
    import os
    import eazy
    import numpy as np
    
    from grizli import utils
    import mastquery.utils
    
    cat = utils.read_catalog('{0}_phot.fits'.format(root))
    filters = []
    for c in cat.meta:
        if c.endswith('_ZP'):
            filters.append(c.split('_ZP')[0].lower())
    
    if get_external_photometry:
        print('Get external photometry from Vizier')
        try:
            ext = get_external_catalog(cat, external_limits=external_limits,
                                       timeout=external_timeout,
                                       sys_err=external_sys_err)
            for c in ext.colnames:
                if c not in cat.colnames:
                    cat[c] = ext[c]
            
            for k in ext.meta:
                cat.meta[k] = ext.meta[k]
        except:
            print(' - External catalog FAILED')
            pass
        
    # Fix: Take flux_auto when flag==0, flux otherwise
    if (total_flux == 'flux_auto_fix') & (total_flux not in cat.colnames):
        flux = cat['flux_auto']*1.
        flagged = (cat['flag'] > 0)
        flux[flagged] = cat['flux'][flagged]
        cat['flux_auto_fix'] = flux*1.
    
    # Additional auto correction

    cat.meta['TOTALCOL'] = total_flux, 'Column for total flux'
    #cat.meta['HASTOT'] = (auto_corr &  ('tot_corr' in cat.colnames), 'Catalog has full total flux')
    
    apcorr = {}
    for NAPER in range(100):
        if 'APER_{0}'.format(NAPER) not in cat.meta:
            break
    
    if aperture_indices == 'all':
        aperture_indices = range(NAPER)
        
    for i in aperture_indices:
        
        if verbose:
            print('Compute aperture corrections: i={0}, D={1:.2f}" aperture'.format(i, cat.meta['ASEC_{0}'.format(i)]))
            
        if 'flux_aper_{0}'.format(i) in cat.colnames:
            cat['apcorr_{0}'.format(i)] = cat[total_flux]/cat['flux_aper_{0}'.format(i)]
            for f in filters:
                bkgc = '{0}_bkg_aper_{1}'.format(f, i)
                if (bkgc in cat.colnames) & apply_background:
                    bkg = cat[bkgc]
                else:
                    bkg = 0.
                    
                cat['{0}_corr_{1}'.format(f, i)] = (cat['{0}_flux_aper_{1}'.format(f, i)]-bkg)*cat['apcorr_{0}'.format(i)]
                cat['{0}_ecorr_{1}'.format(f, i)] = cat['{0}_fluxerr_aper_{1}'.format(f, i)]*cat['apcorr_{0}'.format(i)]
                
                # mask_thresh = np.percentile(cat['{0}_mask_aper_{1}'.format(f, i)], 95)
                aper_area = np.pi*(cat.meta['APER_{0}'.format(i)]/2)**2
                mask_thresh = aper_area
                
                bad = cat['{0}_mask_aper_{1}'.format(f, i)] > 0.2*mask_thresh
                cat['{0}_corr_{1}'.format(f, i)][bad] = -99
                cat['{0}_ecorr_{1}'.format(f, i)][bad] = -99
                
                tot_col = '{0}_tot_corr'.format(f.lower())
                
                if auto_corr and (tot_col in cat.colnames):   
                    cat['{0}_tot_{1}'.format(f, i)] = cat['{0}_corr_{1}'.format(f, i)]*cat[tot_col]
                    cat['{0}_etot_{1}'.format(f, i)] = cat['{0}_ecorr_{1}'.format(f, i)]*cat[tot_col]

                    cat['{0}_tot_{1}'.format(f, i)][bad] = -99
                    cat['{0}_etot_{1}'.format(f, i)][bad] = -99
                                    
    cat.rename_column('number','id')
    cat['z_spec'] = cat['id']*0.-1
    
    # Spurious sources, sklearn SVM model trained for a single field
    morph_model = os.path.join(os.path.dirname(utils.__file__),
                               'data/sep_catalog_junk.pkl')
                               
    if os.path.exists(morph_model):
        if verbose:
            print('Apply morphological validity class')
        
        from sklearn.externals import joblib
        clf = joblib.load(morph_model)
        X = np.hstack([[cat['peak']/cat['flux'], 
                        cat['cpeak']/cat['peak']]]).T
        
        # Predict labels, which where generated for 
        #    bad_bright, bad_faint, stars, big_galaxies, small_galaxies
        pred = clf.predict_proba(X)
        
        # Should be >~ 0.9 for valid sources, stars & galaxies in "ir" image
        cat['class_valid'] = pred[:,-3:].sum(axis=1) 
        cat['class_valid'].format = '.2f'
    
    cat['dummy_err'] =  10**(-0.4*(8-23.9))
    cat['dummy_flux'] = cat[total_flux] # detection band
    
    if suffix:
        if verbose:
            print('Write {0}_phot{1}.fits'.format(root, suffix))
            
        cat.write('{0}_phot{1}.fits'.format(root, suffix), overwrite=True)
    
    return cat
    
def eazy_photoz(root, force=False, object_only=True, apply_background=True, aper_ix=1, apply_prior=False, beta_prior=True, get_external_photometry=False, external_limits=3, external_sys_err=0.3, external_timeout=300, sys_err=0.05, z_step=0.01, z_min=0.01, z_max=12, total_flux='flux_auto', auto_corr=True, compute_residuals=False, dummy_prior=False, extra_rf_filters=[]):
    
    import os
    import eazy
    import numpy as np
    
    from grizli import utils
    import mastquery.utils
    
    if (os.path.exists('{0}.eazypy.self.npy'.format(root))) & (not force):
        self = np.load('{0}.eazypy.self.npy'.format(root))[0]
        zout = utils.read_catalog('{0}.eazypy.zout.fits'.format(root))
        cat = utils.read_catalog('{0}_phot_apcorr.fits'.format(root))
        return self, cat, zout
        
    trans = {'f098m':201, 'f105w':202, 'f110w':241, 'f125w':203, 'f140w':204, 'f160w':205, 'f435w':233, 'f438w':211, 'f606w':236, 'f625w':237, 'f814w':239, 'f702w':15, 'f555w':235, 'f350lp':339, 'f475w':212, 'f775w':238, 'f850lp':240}
    #trans.pop('f814w')
    
    print('Apply catalog corrections')
    apply_catalog_corrections(root, suffix='_apcorr')
    
    cat = utils.read_catalog('{0}_phot_apcorr.fits'.format(root))
    filters = []
    for c in cat.meta:
        if c.endswith('_ZP'):
            filters.append(c.split('_ZP')[0].lower())
        
    # Translate
    fp = open('zphot.translate','w')
    for f in filters:
        if f in trans:
            fp.write('{0}_tot_{1} F{2}\n'.format(f, aper_ix, trans[f]))
            fp.write('{0}_etot_{1} E{2}\n'.format(f, aper_ix, trans[f]))
    
    fp.write('irac_ch1_flux F18\n')
    fp.write('irac_ch1_err  E18\n')
    
    # For zeropoint
    if dummy_prior:
        fp.write('dummy_flux F205x\n')
        fp.write('dummy_err  E205x\n')
            
    fp.close()
    
    params = {}
    params['CATALOG_FILE'] = '{0}_phot_apcorr.fits'.format(root)
    params['Z_STEP'] = z_step
    params['MAIN_OUTPUT_FILE'] = '{0}.eazypy'.format(root)
    
    params['Z_MAX'] = z_max
    params['MW_EBV'] = mastquery.utils.get_irsa_dust(cat['ra'].mean(), cat['dec'].mean())
    params['PRIOR_ABZP'] = 23.9
    
    params['SYS_ERR'] = sys_err
    params['CAT_HAS_EXTCORR'] = False
    
    # Pick prior filter, starting from reddest
    for f in ['f435w', 'f606w', 'f814w', 'f105w', 'f110w', 'f125w', 'f140w', 'f160w'][::-1]:
        if f in filters:
            if dummy_prior:
                params['PRIOR_FILTER'] = 'dummy_flux'
            else:
                params['PRIOR_FILTER'] = trans[f]
            
            mag = 23.9-2.5*np.log10(cat['{0}_corr_{1}'.format(f, aper_ix)])
            break
    #
    if os.path.exists('templates/fsps_full/tweak_fsps_QSF_11_v3_noRed.param.fits'):
        params['TEMPLATES_FILE'] = 'templates/fsps_full/tweak_fsps_QSF_11_v3_noRed.param'
    else:
        params['TEMPLATES_FILE'] = 'templates/fsps_full/tweak_fsps_QSF_12_v3.param'

    zpfile = None
    load_products = False

    if (not os.path.exists('FILTER.RES.latest') or 
        not os.path.exists('templates')):
        try:
            # should work with eazy-py >= 0.2.0
            eazy.symlink_eazy_inputs(path=None)
        except:
            print("""
The filter file `FILTER.RES.latest` and `templates` directory were not
found in the working directory and the automatic command to retrieve them 
failed: 
    
    >>> import eazy; eazy.symlink_eazy_inputs(path=None)  

Run it with `path` pointing to the location of the `eazy-photoz` repository.""")
            return False
            
    self = eazy.photoz.PhotoZ(param_file=None, translate_file='zphot.translate', zeropoint_file=zpfile, params=params, load_prior=True, load_products=load_products)
    
    if object_only:
        return self
        
    idx = np.arange(self.NOBJ)
    
    #sample = (mag < 27) #& (self.cat['star_flag'] != 1)
    #sample |= (self.cat['z_spec'] > 0)
    sample = np.isfinite(self.cat['id']) #mag)
    
    for iter in range(1+(get_external_photometry & compute_residuals)*1):
        self.fit_parallel(idx[sample], n_proc=10)
        if compute_residuals:
            self.error_residuals()
    
    self.standard_output(prior=apply_prior, beta_prior=beta_prior, extra_rf_filters=extra_rf_filters)
    
    zout = utils.read_catalog('{0}.eazypy.zout.fits'.format(root))
    
    np.save('{0}.eazypy.self.npy'.format(root), [self])
    
    return self, cat, zout
    
def show_from_ds9(ds9, self, zout, **kwargs):
    
    import numpy as np
    
    xy = np.cast[float](ds9.get('pan image').split())
    r = np.sqrt((self.cat['x_image']-xy[0])**2 + (self.cat['y_image']-xy[1])**2)
    
    ix = np.argmin(r)
    print('ID: {0}, r={1:.1f} pix'.format(self.cat['id'][ix], r[ix]))
    print('  z={0:.2f} logM={1:.2f}'.format(zout['z_phot'][ix], np.log10(zout['mass'][ix])))
    
    fig = self.show_fit(self.cat['id'][ix], **kwargs)
    return fig, self.cat['id'][ix], zout['z_phot'][ix]
     
class EazyPhot(object):
    def __init__(self, photoz, grizli_templates=None, zgrid=None, apcorr=None):
        """
        photoz : `~eazypy.photoz.PhotoZ`
        
        apcorr : array
            Aperture correction applied to the photometry to match the 
            grism spectra.  For the internal grizli catalogs, this should 
            generally be something like 
            
            >>> apcorr = 'flux_iso' / 'flux_auto'
            
            
        """
        try:
            from .. import utils
        except:
            from grizli import utils
            
        not_obs_mask =  (photoz.fnu < -90) | (photoz.efnu < 0)
        
        self.zgrid = photoz.zgrid
        
        if apcorr is None:
            self.apcorr = np.ones(photoz.NOBJ)
        else:
            self.apcorr = apcorr
            
        self.flam = photoz.fnu*photoz.to_flam*photoz.zp*photoz.ext_corr
        self.flam[not_obs_mask] = -99
        
        self.eflam = photoz.efnu*photoz.to_flam*photoz.zp**photoz.ext_corr
        self.eflam[not_obs_mask] = -99
        
        #self.rdcat = utils.GTable(photoz.cat['ra','dec'])
        self.ra_cat = photoz.cat['ra'].data
        self.dec_cat = photoz.cat['dec'].data
        
        self.filters = photoz.filters
        self.f_numbers = photoz.f_numbers
        self.param = photoz.param
        
        if grizli_templates is None:
            self.tempfilt = None
        else:
            self.tempfilt = self.initialize_templates(grizli_templates,
                                                      zgrid=zgrid)
            
    def initialize_templates(self, grizli_templates, zgrid=None):
        
        from eazy import templates as templates_module 
        from eazy.photoz import TemplateGrid
        
        if zgrid is None:
            zgrid = self.zgrid
            
        template_list = [templates_module.Template(arrays=(grizli_templates[k].wave, grizli_templates[k].flux), name=k) for k in grizli_templates]
        
        tempfilt = TemplateGrid(zgrid, template_list, RES=self.param['FILTERS_RES'], f_numbers=self.f_numbers, add_igm=True, galactic_ebv=self.param['MW_EBV'], Eb=self.param['SCALE_2175_BUMP'])
        
        return tempfilt
    
    def get_phot_dict(self, ra, dec):
        """
        
        """
        from collections import OrderedDict
        try:
            from .. import utils
        except:
            from grizli import utils
            
        icat = utils.GTable()
        icat['ra'] = [ra]
        icat['dec'] = [dec]
        
        rdcat = utils.GTable()
        rdcat['ra'] = self.ra_cat
        rdcat['dec'] = self.dec_cat
        
        ix, dr = rdcat.match_to_catalog_sky(icat)
        
        phot = OrderedDict()
        apcorr_i = self.apcorr[ix[0]]
        
        phot['flam'] = self.flam[ix[0],:]*1.e-19*apcorr_i
        phot['eflam'] = self.eflam[ix[0],:]*1.e-19*apcorr_i
        phot['filters'] = self.filters
        phot['tempfilt'] = self.tempfilt
        return phot, ix[0], dr[0]
        
def get_external_catalog(phot, filter_file='/usr/local/share/eazy-photoz/filters/FILTER.RES.latest', ZP=23.9, sys_err=0.3, verbose=True, external_limits=3, timeout=300):
    """
    Fetch photometry from vizier
    """
    import numpy as np
    
    import astropy.units as u
    
    try:
        from .. import utils
    except:
        from grizli import utils
    
    from eazy.filters import FilterFile
    res = FilterFile(filter_file)
        
    vizier_catalog = list(utils.VIZIER_VEGA.keys())

    ra = np.median(phot['ra'])
    dec = np.median(phot['dec'])
    dr = np.sqrt((phot['ra']-ra)**2*np.cos(phot['dec']/180*np.pi)**2 + (phot['dec']-dec)**2)*60
    
    radius = 1.5*dr.max() # arcmin
    tabs = utils.get_Vizier_photometry(ra, dec, templates=None, radius=radius*60, vizier_catalog=vizier_catalog, filter_file=filter_file, MW_EBV=0, convert_vega=False, raw_query=True, verbose=True, timeout=timeout)

    extern_phot = utils.GTable()
    N = len(phot)
    
    for t_i in tabs:

        # Match
        if 'RAJ2000' in t_i.colnames:
            other_radec = ('RAJ2000','DEJ2000')
        elif 'RA_ICRS' in t_i.colnames:
            other_radec = ('RA_ICRS','DE_ICRS')
        else:
            other_radec = ('ra','dec')
            
        idx, dr = phot.match_to_catalog_sky(t_i, other_radec=other_radec)
        if (dr < 2*u.arcsec).sum() == 0:
            continue
        
        tab = t_i[dr < 2*u.arcsec]
        idx = idx[dr < 2*u.arcsec]
            
        # Downweight PS1 if have SDSS
        # if (tab.meta['name'] == PS1_VIZIER) & (SDSS_DR12_VIZIER in viz_tables):
        #     continue
        #     err_scale = 1
        # else:
        #     err_scale = 1
        err_scale = 1
        
        if (tab.meta['name'] == utils.UKIDSS_LAS_VIZIER):
            flux_scale = 1.33
        else:
            flux_scale = 1.

        convert_vega = utils.VIZIER_VEGA[tab.meta['name']]
        bands = utils.VIZIER_BANDS[tab.meta['name']]

        #if verbose:
        #    print(tab.colnames)

        #filters += [res.filters[res.search(b, verbose=False)[0]] for b in bands]

        to_flam = 10**(-0.4*(48.6))*3.e18 # / pivot(Ang)**2
        
        for ib, b in enumerate(bands):
            f_number = res.search(b, verbose=False)[0]+1
            filt = res.filters[f_number-1]
            #filters.append(filt)

            if convert_vega:
                to_ab = filt.ABVega()
            else:
                to_ab = 0.
            
            fcol, ecol = bands[b]
            #pivot.append(filt.pivot())
            fnu = 10**(-0.4*(tab[fcol]+to_ab-ZP))
            efnu = tab[ecol]*np.log(10)/2.5*fnu*err_scale
            
            efnu = np.sqrt(efnu**2+(sys_err*fnu)**2)
            
            fnu.fill_value = -99
            efnu.fill_value = -99
            
            comment = 'Filter {0} from {1} (N={2})'.format(bands[b][0], t_i.meta['name'], len(idx))
            if verbose:
                print(comment)
            
            if ((~efnu.mask).sum() > 4) & (external_limits > 0):
                fill = np.percentile(efnu.data[~efnu.mask], [90])
                efill = external_limits * fill
            else:
                fill = -99
                efill = -99
                
            extern_phot.meta['F{0}'.format(f_number)] = b, comment
            extern_phot['F{0}'.format(f_number)] = fill*np.ones(N)
            extern_phot['F{0}'.format(f_number)][idx] = fnu.filled()
            
            extern_phot['E{0}'.format(f_number)] = efill*np.ones(N)
            extern_phot['E{0}'.format(f_number)][idx] = efnu.filled()
            
    return extern_phot
    
########### Selecting objects
def select_objects():
    from grizli.pipeline import photoz
    import numpy as np
    
    total_flux = 'flux_auto_fix'
    total_flux = 'flux_auto' # new segmentation masked SEP catalogs
    
    self, cat, zout = photoz.eazy_photoz(root, object_only=False, apply_prior=False, beta_prior=True, aper_ix=1, force=True, get_external_photometry=False, compute_residuals=False, total_flux=total_flux)

    flux = self.cat[total_flux]*1.
    hmag = 23.9-2.5*np.log10(cat['f160w_tot_2'])

    # Reddest HST band
    lc_clip = self.lc*1
    lc_clip[self.lc > 1.55e4] = 0 # in case ground-based / WISE red band
    ixb = np.argmax(lc_clip)
    sn_red = self.cat[self.flux_columns[ixb]]/self.cat[self.err_columns[ixb]]

    grad = np.gradient(self.zgrid)
    cumpz = np.cumsum(self.pz*grad, axis=1); cumpz = (cumpz.T/cumpz[:,-1]).T

    chi2 = (self.chi_best / self.nusefilt)

    iz6 = np.where(self.zgrid > 6.0)[0][0]
    iz7 = np.where(self.zgrid > 7)[0][0]
    iz8 = np.where(self.zgrid > 8)[0][0]
    iz9 = np.where(self.zgrid > 9)[0][0]

    highz_sel = (hmag < 27.5) & (self.cat['class_valid'] > 0.8)
    #highz_sel |= (cumpz[:,iz6] < 0.3) & (self.cat['flux_radius'] > 2.5) 
    #highz_sel &= (self.cat['flux_radius'] > 2.5) 
    highz_sel &= chi2 < 3
    highz_sel &= (sn_red > 5)
    highz_sel &= self.nusefilt >= 3
    
    flux_ratio = (cat['f160w_flux_aper_3'] - cat['f160w_bkg_aper_3'])/(cat['f160w_flux_aper_0'] - cat['f160w_bkg_aper_0'])
    flux_ratio /= cat.meta['APER_3']**2/cat.meta['APER_0']**2
    
    if False:
        sel = highz_sel
        so = np.argsort(cumpz[sel,iz7]); ids = self.cat['id'][sel][so]; i=-1
        so = np.argsort(flux_ratio[sel])[::-1]; ids = self.cat['id'][sel][so]; i=-1
        
    highz_sel &= (cumpz[:,iz6] > 0) & (flux_ratio < 0.45) & ((cumpz[:,iz6] < 0.3) | (cumpz[:,iz7] < 0.3) | (((cumpz[:,iz8] < 0.4) |  (cumpz[:,iz9] < 0.5)) & (flux_ratio < 0.5))) 

    # Big objects likely diffraction spikes
    # big = (self.cat['flux_radius'] > 10)
    # highz_sel &= ~big
    #flux_ratio = self.cat['flux_aper_0']/self.cat['flux_aper_2']

    sel = highz_sel
    so = np.argsort(hmag[sel]); ids = self.cat['id'][sel][so]; i=-1

    # Red 
    uv = -2.5*np.log10(zout['restU']/zout['restV'])
    red_sel = ((zout['z160'] > 1.) & (uv > 1.5)) | ((zout['z160'] > 1.5) & (uv > 1.1))
    red_sel &= (self.zbest < 4) & (hmag < 22)# & (~hmag.mask) 
    red_sel &= (zout['mass'] > 10**10.5) #& (self.cat['class_valid'] > 0.8)
    red_sel &= (self.cat['flux_radius'] > 2.5)
    red_sel &= (zout['restV']/zout['restV_err'] > 3)
    red_sel &= (chi2 < 3)
    #red_sel &= (sn_red > 20)

    sel = red_sel

    so = np.argsort(hmag[sel]); ids = self.cat['id'][sel][so]; i=-1
    
    ds9 = None
    
    for j in self.idx[sel][so]:
        id_j, ra, dec = self.cat['id', 'ra', 'dec'][j]

        # Photo-z
        fig, data = self.show_fit(id_j, ds9=ds9, show_fnu=True) #highz_sel[j])    
        lab = '{0} {1}\n'.format(root, id_j)
        lab += 'H={0:.1f} z={1:.1f}\n'.format(hmag[j], self.zbest[j])
        lab += 'U-V={0:.1f}, logM={1:4.1f}'.format(uv[j], np.log10(zout['mass'][j]))

        ax = fig.axes[0]
        ax.text(0.95, 0.95, lab, ha='right', va='top', transform=ax.transAxes, fontsize=9, bbox=dict(facecolor='w', edgecolor='None', alpha=0.5))
        yl = ax.get_ylim()
        ax.set_ylim(yl[0], yl[1]*1.1)

        fig.savefig('{0}_{1:05d}.eazy.png'.format(root, id_j), dpi=70)
        plt.close()

        # Cutout
        #from grizli_aws.aws_drizzle import drizzle_images
        
        #rgb_params = {'output_format': 'png', 'output_dpi': 75, 'add_labels': False, 'show_ir': False, 'suffix':'.rgb'}
        rgb_params = None

        #aws_bucket = 's3://grizli/SelectedObjects/'
        aws_bucket = None

        label = '{0}_{1:05d}'.format(root, id_j)
        if not os.path.exists('{0}.rgb.png'.format(label)):
            drizzle_images(label=label, ra=ra, dec=dec, pixscale=0.06, size=8, pixfrac=0.8, theta=0, half_optical_pixscale=False, filters=['f160w', 'f814w', 'f140w', 'f125w', 'f105w', 'f110w', 'f098m', 'f850lp', 'f775w', 'f606w', 'f475w'], remove=False, rgb_params=rgb_params, master='grizli-jan2019', aws_bucket=aws_bucket)

        show_all_thumbnails(label=label, filters=['f775w','f814w','f098m','f105w','f110w','f125w','f140w','f160w'], scale_ab=np.clip(hmag[j],19,22), close=True)

############
def show_all_thumbnails(label='j022708p4901_00273', filters=['f775w','f814w','f098m','f105w','f110w','f125w','f140w','f160w'], scale_ab=21, close=True):
    """
    Show individual filter and RGB thumbnails
    """
    from astropy.visualization import make_lupton_rgb
    from PIL import Image
    
    ims = {}
    for filter in filters:
        drz_files = glob.glob('{0}-{1}*_dr*sci.fits'.format(label, filter))
        if len(drz_files) > 0:
            im = pyfits.open(drz_files[0])
            ims[filter] = im
            
    slx, sly, filts, fig = auto_script.field_rgb(root=label, xsize=4, output_dpi=None, HOME_PATH=None, show_ir=False, pl=1, pf=1, scl=1, rgb_scl=[1, 1, 1], ds9=None, force_ir=False, filters=ims.keys(), add_labels=False, output_format='png', rgb_min=-0.01, xyslice=None, pure_sort=False, verbose=True, force_rgb=None, suffix='.rgb', scale_ab=scale_ab)
    if close:
        plt.close()
    
    rgb = np.array(Image.open('{0}.rgb.png'.format(label)))
    
    NX = (len(filters)+1)
    fig = plt.figure(figsize=[1.5*NX,1.5])
    ax = fig.add_subplot(1,NX,NX)
    ax.imshow(rgb, origin='upper', interpolation='nearest')
    ax.text(0.5, 0.95, label, ha='center', va='top', transform=ax.transAxes, fontsize=7, bbox=dict(facecolor='w', edgecolor='None', alpha=0.5))
    
    for i, filter in enumerate(filters):
        if filter in ims:
            zp_i = utils.calc_header_zeropoint(ims[filter], ext=0)
            scl = 10**(-0.4*(zp_i-5-scale_ab))
            img = ims[filter][0].data*scl
            image = make_lupton_rgb(img, img, img, stretch=0.1, minimum=-0.01)
            
            ax = fig.add_subplot(1,NX,i+1)
            ax.imshow(255-image, origin='lower', interpolation='nearest')
            
            ax.text(0.5, 0.95, filter, ha='center', va='top', transform=ax.transAxes, fontsize=7, bbox=dict(facecolor='w', edgecolor='None', alpha=0.5))
    
    for ax in fig.axes:
        ax.set_xticklabels([])
        ax.set_yticklabels([])
        ax.set_xticks([])
        ax.set_yticks([])
    
    fig.tight_layout(pad=0.1)
    
    fig.savefig('{0}.thumb.png'.format(label))
    if close:
        plt.close()
        
def iyj():

    flux = 'corr'
    err = 'ecorr'
    ap = 0
    
    iflux = np.maximum(self.cat['f814w_{0}_{1}'.format(flux, ap)], self.cat['f814w_{0}_{1}'.format(err, ap)])

    iSN = self.cat['f814w_{0}_{1}'.format(flux, ap)]/self.cat['f814w_{0}_{1}'.format(flux, ap)]

    yflux = self.cat['f105w_{0}_{1}'.format(flux, ap)]
    jflux = self.cat['f125w_{0}_{1}'.format(flux, ap)]
    
    iy = -2.5*np.log10(iflux/yflux)
    yj = -2.5*np.log10(yflux/jflux)
    
    sel = (hmag > 22) & (self.cat['f814w_{0}_{1}'.format(flux, ap)] > -90)
    qso = self.cat['id'] == 1072
    plt.scatter(yj[sel], iy[sel], marker='.', color='k', alpha=0.5)
    plt.scatter(yj[highz_sel], iy[highz_sel], marker='s', color='r', alpha=0.5)
    plt.scatter(yj[qso], iy[qso], marker='*', color='r')
    
    red = (yj > 0.5) & (iy > 1.2) & (self.cat['f814w_{0}_{1}'.format(flux, ap)] > -90) & (iSN < 2)

    red = (yj > -0.2) & (iy > 1.9) & (self.cat['f814w_{0}_{1}'.format(flux, ap)] > -90) & (iSN < 2)

    plt.scatter(yj[red], iy[red], marker='.', color='pink')
    
    tf = self.tempfilt.tempfilt
    tiy = -2.5*np.log10(tf[:,:,2]/tf[:,:,0])
    tyj = -2.5*np.log10(tf[:,:,0]/tf[:,:,1])
    lowz = self.zgrid < 6
    plt.plot(tyj[lowz,:], tiy[lowz,:], alpha=0.3, color='orange')
    plt.plot(tyj[~lowz,:], tiy[~lowz,:], alpha=0.4, color='r')
    plt.xlim(-1,3); plt.ylim(-1,10); plt.grid()
    


"""
Utilities for handling JWST file/data formats.

Requires https://github.com/spacetelescope/jwst

"""

import numpy as np

def hdu_to_imagemodel(in_hdu):
    """
    Workaround for initializing a `jwst.datamodels.ImageModel` from a 
    normal FITS ImageHDU that could contain HST header keywords and 
    unexpected WCS definition.
    
    TBD
    
    Parameters
    ----------
    in_hdu : `astropy.io.fits.ImageHDU`
    
    
    Returns
    -------
    img : `jwst.datamodels.ImageModel`
    
    
    """
    from astropy.io.fits import ImageHDU, HDUList
    from astropy.coordinates import ICRS
    
    from jwst.datamodels import util
    import gwcs
        
    hdu = ImageHDU(data=in_hdu.data, header=in_hdu.header)
    
    new_header = strip_telescope_header(hdu.header)
    
    hdu.header = new_header
    
    # Initialize data model
    img = util.open(HDUList([hdu]))
    
    # Initialize GWCS
    tform = gwcs.wcs.utils.make_fitswcs_transform(new_header)
    hwcs = gwcs.WCS(forward_transform=tform, output_frame=ICRS())#gwcs.CelestialFrame())
    sh = hdu.data.shape
    hwcs.bounding_box = ((-0.5, sh[0]-0.5), (-0.5, sh[1]-0.5))
    
    # Put gWCS in meta, where blot/drizzle expect to find it
    img.meta.wcs = hwcs
    
    return img
    
def xxx(header):
    """
    """
    ra, dec = 53.18118642, -27.79096316
    hdu = utils.make_wcsheader(ra=ra, dec=dec, size=10, pixscale=0.06, get_hdu=True)
    out = grizli.jwst.hdu_to_imagemodel(hdu)
    
    from jwst.datamodels import ModelContainer, DrizProductModel
    
    product = DrizProductModel(out.data.shape)
    product.meta.wcs = out.meta.wcs
    
    from jwst.resample import gwcs_blot, gwcs_drizzle
    driz = gwcs_drizzle.GWCSDrizzle(product)#, outwcs=out.meta.wcs)
    
    driz.add_image(blot_data, wcs_model.meta.wcs, xmax=out.data.shape[1], ymax=out.data.shape[0])
    
    from jwst.resample import resample_utils
    from drizzle import util
    
    input_wcs = wcs_model.meta.wcs
    output_wcs = out.meta.wcs
    
    fillval = 'INDEF'
    insci = blot_data
    inwht = None
    xmin = xmax = ymin = ymax = 0
    uniqid = 1
    outsci = driz.outsci*1
    outwht = driz.outwht*1
    outcon = driz.outcon*1
    in_units = 'cps'
    
    from jwst.resample import resample
    
    groups = ModelContainer([wcs_model])
    sampler = resample.ResampleData(groups, output=driz)

def change_header_pointing(header, ra_ref=0., dec_ref=0., pa_v3=0.):
    """
    Update a FITS header for a new pointing (center + roll).  
    
    Parameters
    ----------
    header : `~astropy.io.fits.Header`
        Parent header (must contain `V2_REF`, `V3_REF` keywords).
    
    ra_ref, dec_ref : float
        Pointing center, in decimal degrees, at reference the pixel defined 
        in.
        
    pa_v3 : float
        Position angle of the telescope V3 axis, degrees.
    
    .. warning::
    
    Doesn't update PC keywords based on pa_v3, which would rather have to 
    be computed from the new `gwcs`.

    """
    from jwst.lib.set_telescope_pointing import compute_local_roll
    
    v2_ref = header['V2_REF']
    v3_ref = header['V3_REF']
    
    # Strip units, if any
    args = []
    for v in (pa_v3, ra_ref, dec_ref, v2_ref, v3_ref):
        if hasattr(v, 'value'):
            args.append(v.value)
        else:
            args.append(v)
            
    roll_ref = compute_local_roll(*tuple(args))
    
    new_header = header.copy()
    new_header['XPA_V3'] = args[0]
    new_header['CRVAL1'] = new_header['RA_REF'] = args[1]
    new_header['CRVAL2'] = new_header['DEC_REF'] = args[2]
    new_header['ROLL_REF'] = roll_ref
    return new_header
    
def img_with_wcs(input):
    """
    Open a JWST exposure and apply the distortion model.  
    
    Parameters
    ----------
    input : type
        Anything `jwst.datamodels.util.open` can accept for initialization.
    
    Returns
    -------
    with_wcs : `jwst.datamodels.ImageModel`
        Image model with full `~gwcs` in `with_wcs.meta.wcs`.
        
    """
    from jwst.datamodels import util
    from jwst.assign_wcs import AssignWcsStep

    # from jwst.stpipe import crds_client
    # from jwst.assign_wcs import assign_wcs
    
    # HDUList -> jwst.datamodels.ImageModel
    img = util.open(input)
    
    # AssignWcs to pupulate img.meta.wcsinfo
    step = AssignWcsStep()
    with_wcs = step.process(img)
    
    ## Above should be more robust to get all of the necessary ref files
    #dist_file = crds_client.get_reference_file(img, 'distortion')
    #reference_files = {'distortion': dist_file}
    #with_wcs = assign_wcs.load_wcs(img, reference_files=reference_files)
    
    return with_wcs
    
def strip_telescope_header(header, simplify_wcs=True):
    """
    Strip non-JWST keywords that confuse `jwst.datamodels.util.open`.
    
    Parameters
    ----------
    header : `~astropy.io.fits.Header`
        Input FITS header.
    
    """
    import astropy.wcs as pywcs
    
    new_header = header.copy()
    
    if 'TELESCOP' in new_header:
        if new_header['TELESCOP'] != 'JWST':
            keys =  ['TELESCOP', 'FILTER', 'DETECTOR', 'INSTRUME']
            for key in keys:
                if key in header:
                    new_header.remove(key)
    
    if simplify_wcs:
        # Make simple WCS header
        orig_wcs = pywcs.WCS(new_header)
        new_header = orig_wcs.to_header()  
          
        new_header['EXTNAME'] = 'SCI'
        new_header['RADESYS'] = 'ICRS'
        new_header['CDELT1'] = -new_header['PC1_1']
        new_header['CDELT2'] = new_header['PC2_2']
        new_header['PC1_1'] = -1
        new_header['PC2_2'] = 1

    return new_header

def model_wcs_header(datamodel, get_sip=False, order=4, step=32):
    """
    Make a header with approximate WCS for use in DS9.
    
    Parameters
    ----------
    datamodel : `jwst.datamodels.ImageModel`
        Image model with full `~gwcs` in `with_wcs.meta.wcs`.
    
    get_sip : bool
        If True, fit a `astropy.modeling.models.SIP` distortion model to the 
        image WCS.
    
    order : int
        Order of the SIP polynomial model.
    
    step : int
        For fitting the SIP model, generate a grid of detector pixels every
        `step` pixels in both axes for passing through
        `datamodel.meta.wcs.forward_transform`.
        
    Returns
    -------
    header : '~astropy.io.fits.Header`
        Header with simple WCS definition: CD rotation but no distortion.
        
    """  
    from astropy.io.fits import Header
    from scipy.optimize import least_squares
    
    sh = datamodel.data.shape
    
    try:
        pipe = datamodel.meta.wcs.pipeline[0][1]
        if 'offset_2' in pipe.param_names:
            # NIRISS WCS
            c_x = pipe.offset_2.value
            c_y = pipe.offset_3.value
        else:
            # Simple WCS
            c_x = pipe.offset_0.value
            c_y = pipe.offset_1.value
        
        crpix = np.array([-c_x+1, -c_y+1])
        
    except:
        crpix = np.array(sh)/2.+0.5
    
    crval = datamodel.meta.wcs.forward_transform(crpix[0], crpix[1])
    cdx = datamodel.meta.wcs.forward_transform(crpix[0]+1, crpix[1])
    cdy = datamodel.meta.wcs.forward_transform(crpix[0], crpix[1]+1)
    
    header = Header()
    header['RADESYS'] = 'ICRS'
    header['CTYPE1'] = 'RA---TAN'
    header['CTYPE2'] = 'DEC--TAN'
    
    header['CUNIT1'] = header['CUNIT2'] = 'deg'
    
    header['CRPIX1'] = crpix[0]
    header['CRPIX2'] = crpix[1]

    header['CRVAL1'] = crval[0]
    header['CRVAL2'] = crval[1]
    
    cosd = np.cos(crval[1]/180*np.pi)
    
    header['CD1_1'] = (cdx[0]-crval[0])*cosd
    header['CD1_2'] = (cdy[0]-crval[0])*cosd
    
    header['CD2_1'] = cdx[1]-crval[1]
    header['CD2_2'] = cdy[1]-crval[1]
    
    cd = np.array([[header['CD1_1'], header['CD1_2']], [header['CD2_1'], header['CD2_2']]])
    if not get_sip:
        return header
    
    #### Fit a SIP header to the gwcs transformed coordinates
    v, u = np.meshgrid(np.arange(1,sh[0]+1,step), np.arange(1,sh[1]+1,step))
    x, y = datamodel.meta.wcs.forward_transform(u, v)
    y -= crval[1]
    x = (x-crval[0])*np.cos(crval[1]/180*np.pi)
        
    a_names = []
    b_names = []
    #order = 4
    for i in range(order+1):
        for j in range(order+1):
            ext = '{0}_{1}'.format(i,j)
            if (i+j) > order:
                continue
                
            if ext in ['0_0', '0_1','1_0']:
                continue
                
            a_names.append('A_'+ext)
            b_names.append('B_'+ext)
    
    p0 = np.zeros(4+len(a_names)+len(b_names))
    p0[:4] += cd.flatten()
    
    args = (u.flatten(), v.flatten(), x.flatten(), y.flatten(), crpix, a_names, b_names, cd, 0)
    
    # Fit the SIP coeffs
    fit = least_squares(_objective_sip, p0, jac='2-point', bounds=(-np.inf, np.inf), method='lm', ftol=1e-08, xtol=1e-08, gtol=1e-08, x_scale=1.0, loss='linear', f_scale=1.0, diff_step=None, tr_solver=None, tr_options={}, jac_sparsity=None, max_nfev=1000, verbose=0, args=args, kwargs={})
    
    # Get the results 
    args = (u.flatten(), v.flatten(), x.flatten(), y.flatten(), crpix, a_names, b_names, cd, 1)
    
    cd_fit, a_coeff, b_coeff = _objective_sip(fit.x, *args)
    
    # Put in the header
    for i in range(2):
        for j in range(2):
            header['CD{0}_{1}'.format(i+1, j+1)] = cd_fit[i,j]
    
    header['CTYPE1'] = 'RA---TAN-SIP'
    header['CTYPE2'] = 'DEC--TAN-SIP'
    
    header['A_ORDER'] = order
    for k in a_coeff:
        header[k] = a_coeff[k]
        
    header['B_ORDER'] = order
    for k in b_coeff:
        header[k] = b_coeff[k]
    
    return header
    
def _objective_sip(params, u, v, x, y, crpix, a_names, b_names, cd, ret):
    """
    Objective function for fitting SIP coefficients
    """
    from astropy.modeling import models, fitting
    
    #u, v, x, y, crpix, a_names, b_names, cd = data
    
    cdx = params[0:4].reshape((2,2))
    a_params = params[4:4+len(a_names)]
    b_params = params[4+len(a_names):]
    
    a_coeff = {}
    for i in range(len(a_names)):
        a_coeff[a_names[i]] = a_params[i]
    
    b_coeff = {}
    for i in range(len(b_names)):
        b_coeff[b_names[i]] = b_params[i]
    
    if ret == 1:
        return cdx, a_coeff, b_coeff
        
    sip = models.SIP(crpix=crpix, a_order=4, b_order=4, a_coeff=a_coeff, b_coeff=b_coeff)
    
    fuv, guv = sip(u,v)
    xo, yo = np.dot(cdx, np.array([u+fuv-crpix[0], v+guv-crpix[1]]))
    #dr = np.sqrt((x-xo)**2+(y-yo)**2)*3600.
    dr = np.append(x-xo, y-yo)*3600./0.065
    
    #print(params, np.abs(dr).max())
    
    return dr
    
    
    
    
    
        
        
    
    
    
    
    


"""
Extensions of the `~pyds9.DS9` object
"""

# Extend pyds9.DS9 class
try:
    # Avoid choking if pyds9 not available
    import pyds9
    ds9_obj = pyds9.DS9
except:
    print('NB: Couldn\'t `import pyds9`.  The `grizli.ds9.DS9` object will be broken')
    ds9_obj = object

class DS9(ds9_obj):
    """
    Extend `~pyds9.DS9` object with convenience classes
    """
    def view(self, img, header=None):
        """
        From pysao
        """
        if hasattr(img, 'header'):
            ### FITS HDU
            self.set_np2arr(img.data)
            if not header:
                header = img.header
            self.set("wcs replace", header.tostring())
        else:
            self.set_np2arr(img)
            if header:
                self.set("wcs replace", header.tostring())

    def frame(self, id):
        self.set('frame %d' %(id))
    
    def scale(self, z1, z2):
        self.set('scale limits %f %f' %(z1, z2))
        
    def set_defaults(self, match='image', verbose=False):
        """
        Match frame, set log scale
        """
        commands = """
        xpaset -p ds9 scale log
        xpaset -p ds9 scale limits -0.1 10
        xpaset -p ds9 cmap value 3.02222 0.647552
        xpaset -p ds9 match frames %s 
        xpaset -p ds9 frame lock %s
        xpaset -p ds9 match colorbars 
        xpaset -p ds9 lock colorbar
        xpaset -p ds9 match scales""" %(match, match)
                
        for c in commands.split('\n'):
            if 'xpaset' in c:
                self.set(' '.join(c.split()[3:]))
                if verbose:
                    print (c)
    
    def match(self, match='image'):
        commands = """
        xpaset -p ds9 match frames %s 
        xpaset -p ds9 frame lock %s
        """ %(match, match)
        
        for c in commands.split('\n'):
            if 'xpaset' in c:
                self.set(' '.join(c.split()[3:]))